=head1 NAME
 
MAssSize - Module that extracts mass sizes from free text mammograms

=cut

=head1 VERSION

Version 3

-Adds wire to exclusions
-Since \w matches alphanumerical, replaced by alpha

Returns the measurement set with the highest number of dimensions, up to three dimensions. If multiple, returns the largest one.
E.g.: "abnormality of 3 mm x 4 cm is encountered in left breast. Abnormalities of 4x2x1 cm and 2x5x1 cm in right breast"
returns: 50x20x10 mm

=cut 

package MassSize;

use strict;
use warnings;

BEGIN {
    use Exporter();

    our @ISA = qw(Exporter);
    our @EXPORT = qw(&mssz); # subroutines automatically imported into the caller
    our @EXPORT_OK = qw(); #caller specifies the subroutines to import
    our %EXPORT_TAGS = (); #group related subroutines into categories.
                       #caller can thus import groups eg: TAG => [ qw!name1 name2! ]
    #IMP: In OO modules, export nothing.

    our $VERSION = 2;
}


#better than the one used in ICDM paper. Can catch \.\d, centimeters,
#not followed by nipple.
sub mssz(){
    my $report = shift;
    $report = lc($report); #convert to lower case letters
    my $units = "mm|cm|millimeters|centimeters|millimeter|centimeter";
    my @AoA; #AoA to store the various sets of measurements
    my $word = '((\w|\d(\.\d)?)*?[\s,\-\(\):\\\/\']+?)';

    while($report =~ /
	(?:(?:(?:(\d+\.\d+|\.\d+|\d+)\s+)? #nb $1 followed by
	(?:(\d+)\/(\d+))                   #frac $2, $3
	|(\d+\.\d+|\.\d+|\d+))             #or just nb $4
	(?:\s*(?:[[:alpha:]]+\s+)?($units))?\s*              #unit $5
	[xX\*]\s*)?                        #times

	(?:(?:(?:(\d+\.\d+|\.\d+|\d+)\s+)? #nb $1 followed by
	(?:(\d+)\/(\d+))                   #frac $2, $3
	|(\d+\.\d+|\.\d+|\d+))             #or just nb $4
	(?:\s*(?:[[:alpha:]]+\s+)?($units))?\s*              #unit $5
	[xX\*]\s*)?                        #times

	(?:(?:(\d+\.\d+|\.\d+|\d+)\s+)? #nb $1 followed by
	(?:(\d+)\/(\d+))                #frac $2, $3
	|(\d+\.\d+|\.\d+|\d+))          #or just nb $4
	\s*(?:[[:alpha:]]+\s+)?($units)               #unit $5 must be present
	(?!$word{0,5}? \w*(nipp?l|wire)) #not followed by nipple or wire
	/xg)
    {
    
	my $massSize = 0;
	my @sizes = (0,0,0);

	#process the numbers
	foreach my $i (0..2){
	    no strict 'refs';
	    $sizes[$i] = ${5*$i+4} if defined ${5*$i+4};
	    $sizes[$i] = ${5*$i+1} if defined ${5*$i+1};
	    $sizes[$i] += ${5*$i+2}/${5*$i+3} if defined ${5*$i+2};
	    my $unit = ${5*$i+5}||$15;
	    $sizes[$i] *= 10 if $unit && 
	    ($unit eq 'cm' || $unit eq 'centimeter' || $unit eq 'centimeters');
	}

	@sizes = sort {$b <=> $a} @sizes;
	$AoA[@AoA] = \@sizes; #store the sorted sets of measurements
    }

    #return the largest number of the measurement having the largest set.
    @AoA = sort {$b->[0] <=> $a->[0]} @AoA;
    for(my $j=2; $j >=0; $j--){
	foreach my $i (0..$#AoA){
	    return $AoA[$i] if $AoA[$i][$j] > 0;
	}
    }
    #if no measurements
    return [0,0,0];
}

1;
