#!/usr/bin/perl

=head1 NAME
 
parse - The interface to the Parser module. Parses mammography text field extracts BI-RADS info.

Published in:
Information extraction for clinical data mining: A mammography case study
Nassif, H., Wood, R., Burnside, E. S., Ayvaci, M., Shavlik, J., and Page, D.
9th IEEE International Conference on Data Mining Workshops (ICDMW'09), Miami, pp. 37-42, 2009

=cut

=head1 USAGE

usage: perl parse.pl -f input_file -o output_file -n text_column -r 1 -b 0

       -f: the input file, in a csv format
       -o: the output file. Defaults to [input_file].ext
       -d: the field delimiter. Defaults to tab.
       -n: the column number containing the free text reports to parse. Start counting from zero. Defaults to 21.
       -r: rearranges the csv table and inserts the new fields. Default. Else outputs only the new fields.
       -b: Binary, returns 0/1 in the new fields, instead of counts. Defaults to non binary.

examples:
perl parse.pl -f UCSF_Small.csv
perl parse.pl -f Coded.csv -n 1 -r 1 -b 0
perl parse.pl -f No_descriptors.csv -n 40 -r 1

=cut

use strict;
use warnings;
use Carp;
use Parser;

&main(@ARGV);

sub main{
    my %args = @_;
    my $textFile = $args{-f} || confess "Input a file as: -f file_name";
    my $outputFile = $args{-o} || "$textFile.ext";
    my $fieldDel = $args{-d} || "\t";
    my $reportNb = $args{-n};
    $reportNb = 21 if !defined $reportNb;
    my $rearrange = $args{-r};
    $rearrange = 1 if !defined $rearrange;
    my $binary = $args{-b};
    $binary = 0 if !defined $binary;


    my $parser = Parser->new('-inputFile' => $textFile,
			     '-outputFile' => $outputFile,
			     '-fieldDelimiter' => $fieldDel,
			     '-reportNB' => $reportNb,
			     '-binary' => $binary,
			    );

    $parser->extract('-rearrange' => $rearrange);
}

